#! /bin/sh
# the next line restarts using wish \
exec wish "$0" ${1+"$@"}

# ----------------------------------------------------------------------
# Copyright (c) 2019 Richard Ellis
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation 51
# Franklin Street, Suite 500 Boston, MA 02110-1335
# ----------------------------------------------------------------------

array set contents {clipboard "" PRIMARY "" CLIPBOARD "" SECONDARY ""}

foreach font [font names] {
  font configure $font -size [expr {abs([font configure $font -size])}]
}

frame .f1
text .f1.text
scrollbar .f1.vsb
grid .f1.text .f1.vsb -sticky news
grid columnconfigure .f1 0 -weight 1
grid rowconfigure .f1 0 -weight 1

grid .f1 - - -sticky news 

proc status {args} {
  .f1.text insert end [join $args]\n
  .f1.text see end
}

proc handler {selection type format offset max} {
  status Selection request for $selection at offset=$offset max=$max
  return $::contents($selection)
}

proc timestamp {} {
  set ms [clock milliseconds]
  set s [expr {$ms / 1000}]
  set rem [expr {$ms % 1000}]
  return [clock format $s -format "%Y-%m-%dT%H:%M:%S.$rem"]
}

proc ownership-lost {which} {
  status Ownership of $which lost
}

selection handle -selection PRIMARY   . {handler PRIMARY STRING STRING}
selection handle -selection CLIPBOARD . {handler CLIPBOARD STRING STRING}
selection handle -selection SECONDARY . {handler SECONDARY STRING STRING}

label .l1 -text "Clipboard: "

button .b1 -text "Clear" -command {
  clipboard clear
  status "Clipboard cleared"
  set contents(clipboard) ""
}

button .b2 -text "Insert" -command {
  set contents(clipboard) "clipboard: [timestamp]"
  clipboard clear
  clipboard append -- $contents(clipboard)
  status Inserted string '$contents(clipboard)' into clipboard
}

grid .l1 .b1 .b2 -sticky news 

label .l2 -text "X11 PRIMARY: "

button .b3 -text "Clear" -command {
  selection clear -selection PRIMARY
  status X11 PRIMARY cleared
  set contents(PRIMARY) ""
}

button .b4 -text "Take Ownership" -command {
  set contents(PRIMARY) "X11 PRIMARY: [timestamp]"
  selection clear -selection PRIMARY
  status X11 PRIMARY selection cleared
  selection own -command {ownership-lost PRIMARY} -selection PRIMARY .
  status Owning X11 PRIMARY with content '$contents(PRIMARY)'
}

grid .l2 .b3 .b4 -sticky news

label .l3 -text "X11 CLIPBOARD: "

button .b5 -text "Clear" -command {
  selection clear -selection CLIPBOARD
  status X11 CLIPBOARD cleared
  set contents(CLIPBOARD) ""
}

button .b6 -text "Take Ownership" -command {
  set contents(CLIPBOARD) "X11 CLIPBOARD: [timestamp]"
  selection clear -selection CLIPBOARD
  status X11 CLIPBOARD selection cleared
  selection own -command {ownership-lost CLIPBOARD} -selection CLIPBOARD .
  status Owning X11 CLIPBOARD with content '$contents(CLIPBOARD)'
}

grid .l3 .b5 .b6 -sticky news

label .l4 -text "X11 SECONDARY: "

button .b7 -text "Clear" -command {
  selection clear -selection SECONDARY
  status X11 SECONDARY cleared
  set contents(SECONDARY) ""
}

button .b8 -text "Take Ownership" -command {
  set contents(SECONDARY) "X11 SECONDARY: [timestamp]"
  selection clear -selection SECONDARY
  status X11 SECONDARY selection cleared
  selection own -command {ownership-lost SECONDARY} -selection SECONDARY .
  status Owning X11 SECONDARY with content '$contents(SECONDARY)'
}

grid .l4 .b7 .b8 -sticky news
